// WinTextField.java
// by Natsha Etcode

/*
RELEASE NOTICE
This software is released into the Public Domain.
There are no restrictions on its use or distribution.

DISCLAIMER
        This software and its documentation are provided "AS IS" and
without any expressed or implied warranties whatsoever.  No warranties
as to performance, merchantability, or fitness for a particular
purpose exist.
        The user is advised to test the software thoroughly before
relying on it.  The user must assume the entire risk and liability of
using this software.  In no event shall any person or organization of
people be held responsible for any direct, indirect, consequential or
inconsequential damages or lost profits.
*/

package winifc.widgets;

import netscape.application.*;
import netscape.util.*;

// XXX Need override some methods to handle enable/disable, textHighlightText,
// etc.

public class WinTextField extends TextField {
    boolean hasFocus;
    boolean mouseIsDown;
    Target escTarget;
    String escCommand;
    boolean trackEditability = false;
    View nextSelectView, prevSelectView;

    public WinTextField(int x, int y, int w, int h) {
        super(x, y, w, h);
        setBorder(WinBorder.loweredBezel());
        setFont(WinFont.defaultFont());
        setEnabledColors();
        nextSelectView = null;
        prevSelectView = null;
    }

    public WinTextField() {
        this(0, 0, 0, 0);
    }

    public Color highlightTextColor() {
        return WinColor.textHighlightText;
    }

    public void setTrackEditability(boolean value) {
        trackEditability = value;
    }

    public void setEnabledColors() {
        setBackgroundColor(WinColor.text);
        setSelectionColor(WinColor.textHighlight);
        setCaretColor(WinColor.textText);
        setTextColor(WinColor.textText);
    }

    public void setDisabledColors() {
        setBackgroundColor(WinColor.control);
        setSelectionColor(WinColor.controlHighlight);
        setCaretColor(WinColor.textText);
        setTextColor(WinColor.textInactiveText);
    }

    public void setEditable(boolean aFlag) {
        super.setEditable(aFlag);
        if(!trackEditability)
            return;

        if(aFlag) {
            setEnabledColors();
        } else {
            setDisabledColors();
        }
    }

    // XXX This won't work with multi-line text fields and has many hidden
    // assumptions.

    public void drawViewStringAt(Graphics g, String str, int x, int y) {
        int start, stop, count, width;
        String sub;
        FontMetrics metrics;
        Range range;

        if (!hasSelection() || !hasFocus) {
            g.setColor(textColor());
            g.drawString(str, x, y);
            return;
        }

        range = selectedRange();
        start = range.index;
        stop = start + range.length;
        count = charCount();

        metrics = font().fontMetrics();

        if (start > 0) {
            sub = str.substring(0, start);
            g.setColor(textColor());
            g.drawString(sub, x, y);
            x += metrics.stringWidth(sub);
        }

        sub = str.substring(start, stop);
        g.setColor(highlightTextColor());
        g.drawString(sub, x, y);

        if ((count - stop) > 0) {
            x += metrics.stringWidth(sub);
            sub = str.substring(stop, count);
            g.setColor(textColor());
            g.drawString(sub, x, y);
        }
    }

    public void startFocus() {
        hasFocus = true;
        super.startFocus();
    }

    public void stopFocus() {
        hasFocus = false;
        super.stopFocus();
    }

    // XXX This is a hack to get around a JDK bug where mouse up is not
    // delivered when the user mouse downs in the text field, drags out of the
    // window, and then releases the mouse.

    public boolean mouseDown(MouseEvent e) {
        mouseIsDown = true;
        return super.mouseDown(e);
    }

    public void mouseUp(MouseEvent e) {
        if (mouseIsDown) {
            mouseIsDown = false;
            super.mouseUp(e);
        }
    }

    public void mouseDragged(MouseEvent e) {
        if (mouseIsDown) {
            super.mouseDragged(e);
        }
    }

    public void keyDown(KeyEvent e) {
        MouseEvent fakeUp;

        if (mouseIsDown) {
            fakeUp = new MouseEvent();
            fakeUp.setType(MouseEvent.MOUSE_UP);
            fakeUp.setProcessor(rootView());
            mouseUp(fakeUp);
        }

        // Somehow, the TextField is hammering the CommandKey... IFC code and
        // disallowing the ESCAPE_KEY binding to work on other views.
        // I couldn't figure out where the world was going wrong,
        // so I added this into our version.
        if(e.isEscapeKey()) {
            if(escTarget != null) {
                escTarget.performCommand(escCommand, null);
                return;
            }
        }

        super.keyDown(e);
    }

    public void copy() {
        if (drawableCharacter() == ANY_CHARACTER) {
            super.copy();
        }
    }

    public void cut() {
        if (drawableCharacter() == ANY_CHARACTER) {
            super.cut();
        }
    }

    public void setEscTarget(Target t) { escTarget = t; }
    public void setEscCommand(String c) { escCommand = c; }

    public void setNextView(View view) { nextSelectView = view; invalidateKeyboardSelectionOrder(); }
    public View nextSelectableView() {
        if(nextSelectView != null)
            return nextSelectView;
        return super.nextSelectableView();
    }

    public void setPrevView(View view) { prevSelectView = view; }
    public View previousSelectableView() {
        if(prevSelectView != null)
            return prevSelectView;
        return super.previousSelectableView();
    }

    public void decode(Decoder decoder) throws CodingException {
        super.decode(decoder);
        setFont(WinFont.defaultFont());
        setEnabledColors();
    }
}


